
//
// DrawSaturn.java -- display positions of Saturn and its moons.
//	Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

package Akk.Astro;

import java.awt.*;
import java.applet.*;
import java.util.Date;
import java.awt.event.*;

public class DrawSaturn extends Applet
    implements ActionListener, ItemListener
{
    Saturn saturn;

    StarDate curDate;

    double Scale;

     // Copies of the UI components we maintain:
    Canvas canvas;
    Panel controlPanel;

    public String getAppletInfo() {
	return "Saturn Moon Locator.\nCopyright 2002 by Akkana Peck";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	    { "time", "string",
	      "Time of chart (optional, defaults to current)" },
            { "scale", "Integer",
              "Scale, in km per pixel (default 6000)" },
   	};
	return info;
    }

    public void zoom(int direc)
    {
        if (direc < 0)
            Scale = Scale / 1.5;
        else
            Scale = Scale * 1.5;
        canvas.repaint();
    }

    public void init()
    {
    	String datestring = getParameter("time");
	if (datestring == null)
            try {
                curDate = new StarDate();
                datestring = curDate.toString();
            }
	    catch (IllegalArgumentException e) {
		datestring = "Internal java error: can't get current date!";
            }
	else
            try {
                curDate = new StarDate(datestring);
            }
	    catch (IllegalArgumentException e) {
		datestring = "Can't parse date: " + datestring;
            }

        saturn = new Saturn(curDate);

        String s = getParameter("scale");
        if (s != null)
            Scale = 1. / Double.valueOf(s).doubleValue();
        else
            Scale = 1. / 6000.;

	//
	// Set some layout info:
	//
	setLayout(new BorderLayout());

        controlPanel = new AstroPanel(this);
	add("North", controlPanel);

	canvas = new SaturnCanvas();
	add("Center", canvas);
    }

    public void actionPerformed(ActionEvent e)
    {
        //System.out.println("DrawSaturn action performed: "
        //                   + e.getActionCommand());
        //System.out.println("modifiers: " + e.getModifiers());
        //System.out.println("param string: " + e.paramString());

        if (e.getActionCommand().equals(new String("+")))
            zoom(1);
        else if (e.getActionCommand().equals(new String("-")))
            zoom(-1);
        else if (e.getActionCommand().equals(new String("Time")))
        {
            Container container = this.getParent();
            while (! (container instanceof Frame))
                container = container.getParent();
            CalenDialog d = new CalenDialog((Frame)container, this);
            d.show();
        }
        else if (e.getActionCommand().equals(new String("dateChanged")))
        {
            StarCalendar cal = (StarCalendar)e.getSource();
            curDate = cal.getDate();
            saturn.setDate(cal.getDate());
            canvas.repaint();
        }
    }

    public void itemStateChanged(ItemEvent e)
    {
        System.out.println("item state changed: " + e.getItem().toString());
    }

    public class SaturnCanvas extends Canvas
        implements MouseListener
    {
        Color planetColor;
        Color lablColor;

        int mouseX, mouseY;

        // Size constants:
        final static long RS = 60330;   // Radius of planet in km
        final static long RS1 = 92000;  // Inner radius of inner ring in km
        final static long RS2 = 117500;
        final static long RS3 = 122500; // Inner rad of outer ring
        final static long RS4 = 136500; // Outer radius of outer ring in km
        final static int moonSize = 500;	// a sample moon size

        public SaturnCanvas()
        {
            super();
            addMouseListener(this);
        }

        public void paint(Graphics g)
        {
            int width = size().width;
            int height = size().height;
            int xcenter = width/2;
            int ycenter = height/2;

            //try {

                // Do some color manipulation:
                if (planetColor == null)
                {
                    setBackground(Color.black);
                    setForeground(Color.white);
                    planetColor = Color.white;
                    lablColor = Color.white;
                }

                // Clear
                //g.setColor(getBackground());
                g.clearRect(0, 0, width, height);

                // This grey circle is for more 3D effect on the planet:
                int pw = (int)(RS * Scale);
                int ph = (int)(RS * Scale * .8);
                // ... but it doesn't work yet.
                //g.setColor(Color.gray);
                //g.drawOval(xcenter-pw-1, ycenter-ph-1, (pw+1)*2, (ph+1)*2);

                // draw the rings:
                double ringAspect = Math.sin(saturn.Inclination);
                int r4 = (int)(RS4 * Scale);
                int r3 = (int)(RS3 * Scale);
                int r2 = (int)(RS2 * Scale);
                int r1 = (int)(RS1 * Scale);

                g.setColor(planetColor);
                g.fillOval(xcenter - r4, ycenter - (int)(r4*ringAspect),
                           (int)(r4*2), (int)(r4 * 2 * ringAspect));

                g.setColor(getBackground());
                g.fillOval(xcenter - (int)r3, ycenter - (int)(r3*ringAspect),
                           (int)(r3*2), (int)(r3 * 2 * ringAspect));

                g.setColor(planetColor);
                g.fillOval(xcenter - (int)r2, ycenter - (int)(r2*ringAspect),
                           (int)(r2*2), (int)(r2 * 2 * ringAspect));

                g.setColor(getBackground());
                g.fillOval(xcenter - (int)r1, ycenter - (int)(r1*ringAspect),
                           (int)(r1*2), (int)(r1 * 2 * ringAspect));


                // draw the planet itself:
                g.setColor(planetColor);
                g.fillOval(xcenter-pw, ycenter-ph, pw*2, ph*2);

                // and outline the part of the planet on this side of the rings:
                g.setColor(getBackground());
                g.drawArc(xcenter-pw-1, ycenter-ph-1, (pw+1)*2, (ph+1)*2,
                          30, 120);

                // draw each of the moons:
                int msize = (int)(moonSize * Scale);
                if (msize < 3) msize = 2;
                int whichmoon;
                for (whichmoon=0; whichmoon < saturn.getNumMoons();
                     ++whichmoon)
                {
                    XYCoord coord = saturn.getMoonXY(whichmoon);
                    if ((coord.x != Double.POSITIVE_INFINITY)
                        && (coord.y != Double.POSITIVE_INFINITY))
                    {
                        int x = xcenter + (int)(coord.x * pw) - msize;
                        int y = ycenter + (int)(coord.y * pw) - msize;

                        g.setColor(planetColor);
                        g.fillOval(x, y, msize*2, msize*2);

                        g.setColor(lablColor);
                        int xoff, yoff;
                        if (x < xcenter)
                            xoff = -4 * msize;
                        else
                            xoff = 3*msize;
                        if (y < ycenter)
                            yoff = -2*msize;
                        else
                            yoff = 7*msize;
                        g.drawString(saturn.getMoonLabel(whichmoon),
                                     x+xoff, y+yoff);
                    }
                } // end loop over moons

                g.drawString(curDate.toString(), 5, 17);
                g.drawString("Inclination: "
                         + new Integer((int)(saturn.Inclination*180/Math.PI)),
                             5, 35);
  /*
            } // end "try" clause for successful paint

            catch (IllegalArgumentException e) {
                g.setColor(Color.black);
                g.fillRect(0, 0, width, height);
                g.setColor(Color.yellow);
                g.drawString("Error: can't parse date ",
                             xcenter, ycenter);
            }
  */
        } // end SaturnCanvas paint()

        public void mouseClicked(MouseEvent e) { }

        public void mousePressed(MouseEvent e)
        {
            mouseX = e.getX();
            mouseY = e.getY();
        }

        public void mouseReleased(MouseEvent e)
        {
            //System.out.println("From (" + Integer.toString(mouseX) + ", "
            //                   + Integer.toString(mouseY) + ") to ("
            //                   + Integer.toString(e.getX()) + ", "
            //                   + Integer.toString(e.getY()) + ")");
            if (Math.abs(e.getY() - mouseY) < 12
                && Math.abs(e.getX() - mouseX) > 15)
            {
                //System.out.println("From " + curDate.toString());
                // Horizontal drag: advance/retard the time by an hour.
                int dHrs = ((e.getX() > mouseX) ? 1 : -1);
                if (e.getY()+mouseY > size().height)
                    dHrs = -dHrs;
                curDate.setHours(curDate.getHours() + dHrs);
                //System.out.println("To " + curDate.toString());
                saturn.setDate(curDate);
                repaint();
                return;
            }
            if (e.getX() == mouseX && e.getY() == mouseY)
            {
                // Erase any still-showing moon label.
                Graphics g = getGraphics();
                int whereY = 10;
                g.clearRect(0, size().height-whereY*2,
                            size().width/2, whereY*2);
                
                // See if it's close to any moon:
                int xcenter = size().width/2;
                int ycenter = size().height/2;
                int pw = (int)(RS * Scale);

                int found = -1;
                int dist = 50;   // magic distance^2 threshold
                for (int whichmoon=0; whichmoon < saturn.getNumMoons();
                     ++whichmoon)
                {
                    XYCoord coord = saturn.getMoonXY(whichmoon);
                    if ((coord.x != Double.POSITIVE_INFINITY)
                        && (coord.y != Double.POSITIVE_INFINITY))
                    {
                        int x = xcenter + (int)(coord.x * pw) - mouseX;
                        int y = ycenter + (int)(coord.y * pw) - mouseY;
                        if (x*x + y*y < dist)
                        {
                            dist = x*x + y*y;
                            found = whichmoon;
                        }
                    }
                }
                // Did we find one?
                if (found >= 0)
                {
                    g.setColor(Color.yellow);
                    g.drawString(saturn.getMoonInfo(found),
                                 5, size().height-whereY);
                }
            }
        }

        public void mouseEntered(MouseEvent e) { }
        public void mouseExited(MouseEvent e) { }

        public boolean old_handleEvent(Event event)
        {
            //System.out.println("handle event!");

            return super.handleEvent(event);
        }

    } // end SaturnCanvas class
}

